<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use wv\BabelCache\Util;

class WV8_Settings_Manager {
	protected $config;
	protected $syncer;
	protected $registry;
	protected $fileService;
	protected $settings;

	const REGISTRY_KEY = 'wv-globalsettings';

	public function __construct(sly_Configuration $config, WV8_Settings_Synchronizer $syncer, sly_Registry_Registry $registry, sly_Service_File_Base $fileService) {
		$this->config      = $config;
		$this->syncer      = $syncer;
		$this->registry    = $registry;
		$this->fileService = $fileService;
		$this->settings    = null;
	}

	public function getSettings() {
		if ($this->settings !== null) return $this->settings;

		$rawData  = $this->getCachedSettings();
		$settings = array();

		foreach ($rawData as $key => $config) {
			$setting   = new WV8_Setting($key, $config);
			$namespace = $setting->getNamespace();
			$name      = $setting->getName();

			$settings[$namespace][$name] = $setting;
		}

		return ($this->settings = $settings);
	}

	public function refresh($updateCache = true) {
		$cached  = $this->getCachedSettings();
		$current = $this->getCurrentSettings();

		if ($current !== $cached) {
			$this->synchronize($cached, $current, $updateCache);
		}
	}

	public function rebuild($updateCache = true) {
		$current = $this->getCurrentSettings();
		$this->synchronize(array(), $current, $updateCache);
	}

	protected function getCurrentSettings() {
		return $this->config->get('global_settings', array());
	}

	protected function getCachedSettings() {
		return $this->registry->get(self::REGISTRY_KEY, array());
	}

	protected function synchronize($oldData, $newData, $updateCache) {
		$this->syncer->synchronize($oldData, $newData);

		if ($updateCache) {
			$this->writeSettingsCache($newData);
		}
	}

	protected function writeSettingsCache($data) {
		$this->registry->set(self::REGISTRY_KEY, $data);
	}

	public function storeValues(WV8_Setting $setting, $values) {
		$name      = $setting->getName();
		$namespace = $setting->getNamespace();
		$table     = $setting->isLocal() ? 'local_wv8_values' : 'wv8_values';
		$encoded   = WV8_Util::encodeValues($values);
		$sql       = WV_SQL::getInstance();

		$sql->query('UPDATE ~'.$table.' SET `values` = ? WHERE `namespace` = ? AND `name` = ?', array($encoded, $namespace, $name), '~');

		// store the new data in cache
		$cache   = sly_Core::cache();
		$ns      = 'globalsettings.values';
		$key     = md5(md5($namespace).md5($name));
		$setting = array('params' => $setting->getParams(), 'datatype' => $setting->getDatatype(), 'raw' => $values);

		$cache->set($ns, $key, $setting);
	}
}
