<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Value;

use wv\metainfo\Metainfo;

class Service implements \sly_ContainerAwareInterface {
	protected $table = 'wv2_meta';
	protected $container;

	public function setContainer(\sly_Container $container = null) {
		$this->container = $container;
	}

	/**
	 * inserts a metavalue
	 *
	 * @param int    $objectID
	 * @param int    $clangID
	 * @param int    $revision
	 * @param int    $objectType
	 * @param string $metainfo
	 * @param string $value
	 */
	public function insert($objectID, $clangID, $revision, $objectType, $metainfo, $value) {
		$this->container->getPersistence()->insert($this->table, array(
			'object_id' => (int) $objectID,
			'clang'     => (int) $clangID,
			'revision'  => (int) $revision,
			'metainfo'  => trim($metainfo),
			'meta_type' => (int) $objectType,
			'value'     => Service::trim($value)
		));
	}

	public function insertObject($objectID, $clangID, $revision, $objectType, array $metadata) {
		$persistence = $this->container->getPersistence();
		$table       = $this->table;

		$persistence->transactional(
			function () use ($objectID, $clangID, $revision, $objectType, $metadata, $persistence, $table) {
				foreach ($metadata as $metainfo => $value) {
					$persistence->insert($table, array(
						'object_id' => (int) $objectID,
						'clang'     => (int) $clangID,
						'revision'  => (int) $revision,
						'metainfo'  => trim($metainfo),
						'meta_type' => (int) $objectType,
						'value'     => Service::trim($value)
					));
				}
			}
		);

		Metainfo::clearDataCache();
	}

	public function insertArticle(\sly_Model_Article $article, $metainfo, $value) {
		$this->insert($article->getId(), $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE, $metainfo, $value);
	}

	public function insertCategory(\sly_Model_Category $category, $metainfo, $value) {
		$this->insert($category->getId(), $category->getClang(), $category->getRevision(), Metainfo::TYPE_CATEGORY, $metainfo, $value);
	}

	public function insertMedium(\sly_Model_Medium $medium, $clangID, $metainfo, $value) {
		$this->insert($medium->getId(), $clangID, Metainfo::TYPE_MEDIUM, $metainfo, $value);
	}

	public function insertUser(\sly_Model_User $user, $clangID, $metainfo, $value) {
		$this->insert($user->getId(), $clangID, Metainfo::TYPE_USER, $metainfo, $value);
	}

	public function insertArticleData(\sly_Model_Article $article, $metadata) {
		$this->insertObject($article->getId(), $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE, $metadata);
	}

	public function insertCategoryData(\sly_Model_Category $category, $metadata) {
		$this->insertObject($category->getId(), $category->getClang(), $category->getRevision(), Metainfo::TYPE_CATEGORY, $metadata);
	}

	public function insertMediumData(\sly_Model_Medium $medium, $clangID, $metadata) {
		$this->insertObject($medium->getId(), $clangID, $medium->getRevision(), Metainfo::TYPE_MEDIUM, $metadata);
	}

	public function insertUserData(\sly_Model_User $user, $clangID, $metadata) {
		$this->insertObject($user->getId(), $clangID, $user->getRevision(), Metainfo::TYPE_USER, $metadata);
	}

	/***************************************************************************\
	  SELECT
	\***************************************************************************/

	public function findObjectData($objectID, $clang, $revision, $objectType) {
		$db    = $this->container->getPersistence();
		$where = array(
			'object_id' => (int) $objectID,
			'revision'  => (int) $revision,
			'clang'     => (int) $clang,
			'meta_type' => (int) $objectType
		);
		$result = array();

		$db->select($this->table, 'metainfo, value', $where);

		foreach ($db->all() as $row) {
			$metainfo = $row['metainfo'];
			$value    = $row['value'];

			$result[$metainfo] = $value;
		}

		return $result;
	}

	public function findArticleData(\sly_Model_Article $article) {
		return $this->findObjectData($article->getId(), $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE);
	}

	public function findCategoryData(\sly_Model_Category $category) {
		return $this->findObjectData($category->getId(), $category->getClang(), $category->getRevision(), Metainfo::TYPE_CATEGORY);
	}

	public function findMediumData(\sly_Model_Medium $medium, $clangID) {
		return $this->findObjectData($medium->getId(), $clangID, $medium->getRevision(), Metainfo::TYPE_MEDIUM);
	}

	public function findUserData(\sly_Model_User $user, $clangID) {
		return $this->findObjectData($user->getId(), $clangID, $user->getRevision(), Metainfo::TYPE_USER);
	}

	public function findMetainfoData($metainfo, $objectType, $clangID = null, $distinct = false) {
		$db     = $this->container->getPersistence();
		$where  = array('metainfo' => $metainfo, 'meta_type' => (int) $objectType);
		$result = array();

		if ($clangID !== null) {
			$where['clang'] = (int) $clangID;
		}

		$db->select($this->table, 'object_id, clang, value', $where, $distinct ? array('value') : null);

		foreach ($db->all() as $row) {
			$clang    = (int) $row['clang'];
			$objectID = (int) $row['object_id'];
			$value    = $row['value'];

			if ($distinct) {
				$result[] = $value;
			}
			else {
				if ($clangID === null) {
					$result[$clang][$objectID] = $value;
				}
				else {
					$result[$objectID] = $value;
				}
			}
		}

		return $result;
	}

	/***************************************************************************\
	  UPDATE
	\***************************************************************************/

	public function update($objectID, $clangID, $revision, $objectType, $metainfo, $value, $clearCache = true) {
		$this->container->getPersistence()->update($this->table, array('value' => Service::trim($value)), array(
			'object_id' => (int) $objectID,
			'clang'     => (int) $clangID,
			'revision'  => (int) $revision,
			'metainfo'  => trim($metainfo),
			'meta_type' => (int) $objectType
		));

		$clearCache && Metainfo::clearDataCache();
	}

	public function updateArticle(\sly_Model_Article $article, $metainfo, $value, $clearCache = true) {
		$this->updateArticleData($article, array($metainfo => $value), $clearCache);
	}

	public function updateCategory(\sly_Model_Category $category, $metainfo, $value, $clearCache = true) {
		$this->updateCategoryData($category, array($metainfo => $value), $clearCache);
	}

	public function updateMedium(\sly_Model_Medium $medium, $clangID, $metainfo, $value, $clearCache = true) {
		$this->updateMediumData($medium, $clangID, array($metainfo => $value), $clearCache);
	}

	public function updateUser(\sly_Model_User $user, $clangID, $metainfo, $value, $clearCache = true) {
		$this->updateUserData($user, $clangID, array($metainfo => $value), $clearCache);
	}

	public function updateObjectData($objectID, $clangID, $revision, $objectType, array $newMetadata, $clearCache = true) {
		$persistence = $this->container->getPersistence();
		$table       = $this->table;

		$persistence->transactional(
			function () use ($objectID, $clangID, $revision, $objectType, $newMetadata, $persistence, $table) {
				foreach ($newMetadata as $metainfo => $value) {
					$persistence->update($table,
						array('value' => Service::trim($value)),
						array(
							'object_id' => (int) $objectID,
							'clang'     => (int) $clangID,
							'revision'  => (int) $revision,
							'metainfo'  => trim($metainfo),
							'meta_type' => (int) $objectType
						)
					);
				}
			}
		);

		$clearCache && Metainfo::clearDataCache();
	}

	public function updateArticleData(\sly_Model_Article $article, $values, $clearCache = true) {
		$id       = $article->getId();
		$clang    = $article->getClang();
		$revision = $article->getRevision();

		$this->updateObjectData($id, $clang, $revision, Metainfo::TYPE_ARTICLE, $values, $clearCache);

		$params = array(
			'article_id' => $id,
			'clang'      => $clang,
			'revision'   => $revision,
			'values'     => $values
		);

		$this->container->getDispatcher()->notify('WV2_ARTICLE_META_UPDATED', true, $params);
	}

	public function updateCategoryData(\sly_Model_Category $category, $newMetadata, $clearCache = true) {
		$id       = $category->getId();
		$clang    = $category->getClang();
		$revision = $category->getRevision();

		$this->updateObjectData($id, $clang, $revision, Metainfo::TYPE_CATEGORY, $newMetadata, $clearCache);

		$params = array(
			'category_id' => $id,
			'clang'       => $clang,
			'revision'    => $revision,
			'values'      => $newMetadata
		);

		$this->container->getDispatcher()->notify('WV2_CATEGORY_META_UPDATED', true, $params);
	}

	public function updateMediumData(\sly_Model_Medium $medium, $clangID, $newMetadata, $clearCache = true) {
		$id       = $medium->getId();
		$revision = $medium->getRevision();

		$this->updateObjectData($id, $clangID, $revision, Metainfo::TYPE_MEDIUM, $newMetadata, $clearCache = true);

		$params = array(
			'file_id'  => $id,
			'clang'    => $clangID,
			'revision' => $revision,
			'values'   => $newMetadata
		);

		$this->container->getDispatcher()->notify('WV2_MEDIUM_META_UPDATED', true, $params);
	}

	public function updateUserData(\sly_Model_User $user, $clangID, $newMetadata, $clearCache = true) {
		$id       = $user->getId();
		$revision = $user->getRevision();

		$this->updateObjectData($id, $clangID, $revision, Metainfo::TYPE_USER, $newMetadata, $clearCache = true);

		$params = array(
			'file_id'  => $id,
			'clang'    => $clangID,
			'revision' => $revision,
			'values'   => $newMetadata
		);

		$this->container->getDispatcher()->notify('WV2_USER_META_UPDATED', true, $params);
	}

	/**
	 * delete metavalues
	 *
	 * @param int    $objectID    null for all objectids
	 * @param int    $clangID     null for all clangs
	 * @param int    $revision    null for all revisions
	 * @param int    $objectType  null for all types
	 * @param string $metainfo	   null for all metainfos
	 */
	public function delete($objectID = null, $clangID = null, $revision = null, $objectType = null, $metainfo = null) {
		if ($objectID !== null) {
			$where['object_id'] = (int) $objectID;
		}

		if ($clangID !== null) {
			$where['clang'] = (int) $clangID;
		}

		if ($revision !== null) {
			$where['revision'] = (int) $revision;
		}

		if ($objectType !== null) {
			$where['meta_type'] = (int) $objectType;
		}

		if ($metainfo !== null) {
			$where['metainfo'] = (string) $metainfo;
		}

		$this->container->getPersistence()->delete($this->table, $where);
	}

	public function deleteObject($objectID, $objectType, $clangID = null, $revision = null) {
		$this->delete($objectID, $clangID, $revision, $objectType, null);
	}

	public function deleteArticle(\sly_Model_Article $article) {
		$this->deleteObject($article->getId(), Metainfo::TYPE_ARTICLE, $article->getClang(), $article->getRevision());
	}

	public function deleteCategory(\sly_Model_Category $category) {
		$this->deleteObject($category->getId(), Metainfo::TYPE_CATEGORY, $category->getClang(), $category->getRevision());
	}

	public function deleteMedium(\sly_Model_Medium $medium) {
		$this->deleteObject($medium->getId(), Metainfo::TYPE_MEDIUM);
	}

	public function deleteUser(\sly_Model_User $user) {
		$this->deleteObject($user->getId(), Metainfo::TYPE_USER);
	}

	public function deleteMetainfo($metainfo, $objectType, $clangID = null) {
		$this->delete(null, $clangID, null, $objectType, $metainfo);
	}

	public function deleteClang($clangID) {
		$this->delete(null, $clangID);
	}

	public function deleteProperty(\wv\metainfo\Property\Property $property, $clangID = null) {
		$this->deleteMetainfo($property->getName(), $property->getType(), $clangID);
	}

	/***************************************************************************\
	  COPY
	\***************************************************************************/

	public function copy(array $source, array $destination, $objectType) {
		if ($source === $destination) {
			return;
		}

		if ($objectType !== null) {
			$source['meta_type']      = (int) $objectType;
			$destination['meta_type'] = (int) $objectType;
		}

		//$this->container->getPersistence()->transactional(array($this, '_copy'), array($source, $destination, $objectType));
		$persistence = $this->container->getPersistence();
		$table       = $this->table;

		$persistence->transactional(
			function () use ($source, $destination, $persistence, $table) {
				$persistence->delete($table, $destination);
				$persistence->select($table, '*', $source);
				$sourceData = $persistence->all();

				// insert the data as new rows for the destination
				foreach ($sourceData as $row) {
					foreach ($destination as $prop => $value) {
						$row[$prop] = $value;
					}

					$persistence->insert($table, $row);
				}
			}
		);
	}

	public static function trim($value) {
		if (is_float($value) || is_int($value))
			return $value;
		return trim($value);
	}
}
