<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Cache;

use sly_Core;
use sly_DB_PDO_Persistence;
use sly_Service_Article;
use WV8_Settings;
use wv\realurl2\Exception;
use wv\realurl2\RealURL2;
use wv\realurl2\Util;
use WV_Sally;

/**
 * @since  v4.0
 * @author Christoph
 */
class Runtime {
	protected $articleData = array();
	protected $settings    = array();
	protected $urls        = array();
	protected $db          = null;
	protected $service     = null;

	public function __construct(sly_DB_PDO_Persistence $db, sly_Service_Article $service) {
		$this->db      = $db;
		$this->service = $service;
	}

	public function prewarmAll($clang = null) {
		$this->prewarmArticleData($clang);
		$this->prewarmSettings($clang);
	}

	public function prewarmForArticle($id, $clang, $path = null) {
		$this->prewarmSettings($clang);
		$this->prewarmForSiteStartArticle($clang);
		$this->prewarmForArticlePath($clang, $path, $id);
	}

	public function prewarmForArticlePath($clang, $path, $articleID = null) {
		$path = Util::decodePath($path);

		if ($articleID !== null) {
			$path[] = $articleID;
		}

		if (!empty($path)) {
			$this->prewarmArticleData($clang, $path);
		}
	}

	public function prewarmForSiteStartArticle($clang) {
		$articles = array(sly_Core::getSiteStartArticleId());
		$this->prewarmArticleData($clang, $articles);
	}

	public function prewarmArticleData($clang, array $articles = null) {
		// fetch all requested articles in their online or latest revisions

		$where  = $clang === null ? array() : array('clang' => $clang);
		$lookup = $where;

		$lookup['namespace'] = RealURL2::URL_NAMESPACE;

		if ($articles !== null && !empty($articles)) {
			$where['id']          = $articles;
			$lookup['article_id'] = $articles;
		}

		$order    = 'id ASC, clang ASC';
		$articles = $this->service->find($where, null, $order, null, null, null, sly_Service_Article::FIND_REVISION_BEST);

		// create a look-up table for already generated URLs

		$urls = array();

		$this->db->select('local_realurl2', 'article_id, clang, revision, url', $lookup);

		foreach ($this->db->all() as $row) {
			$key        = $row['article_id'].'_'.$row['clang'].'_'.$row['revision'];
			$urls[$key] = $row['url'];
		}

		// transform article list into a more usable structure

		foreach ($articles as $article) {
			$id       = $article->getId();
			$clang    = $article->getClang();
			$revision = $article->getRevision();
			$isCat    = $article->isStartArticle();
			$path     = Util::decodePath($article->getPath());
			$key      = $id.'_'.$clang.'_'.$revision;
			$url      = isset($urls[$key]) ? $urls[$key] : null;

			$this->articleData[$clang][$id] = array(
				'id'       => $id,
				'clang'    => $clang,
				'revision' => $revision,
				'name'     => $isCat ? $article->getCatName() : $article->getName(),
				're_id'    => $article->getParentId(),
				'is_cat'   => $isCat,
				'path'     => $path,
				'type'     => $article->getType(),
				'fragment' => $article->getExtendedValue('wv24_fragment', ''),
				'skipped'  => (boolean) $article->getExtendedValue('wv24_skip', false),
				'url'      => $url
			);
		}
	}

	public function prewarmSettings($clang) {
		if ($clang === null) {
			$data = WV8_Settings::getValues('realurl2', WV_Sally::CLANG_ALL);

			foreach ($data as $name => $settings) {
				foreach ($settings as $clang => $value) {
					if ($name === 'unique_types') {
						$value = array_keys($value);
					}

					$this->settings[$clang][$name] = $value;
				}
			}
		}
		else {
			$clang = (int) $clang;
			$data  = WV8_Settings::getValues('realurl2', $clang);

			foreach ($data as $name => $value) {
				if ($name === 'unique_types') {
					$value = array_keys($value);
				}

				$this->settings[$clang][$name] = $value;
			}
		}
	}

	public function getSetting($key, $clang) {
		if (!isset($this->settings[$clang])) {
			$this->prewarmSettings($clang);
		}

		$settings = $this->settings[$clang];

		// be careful with NULL values
		if (!array_key_exists($key, $settings)) {
			throw new Exception('Unknown setting name "'.$key.'" given.');
		}

		return $settings[$key];
	}

	public function getDefaultFragment($clang) {
		return $this->getSetting('default_fragment', $clang);
	}

	public function getArticleData($articleID, $clang, $key = null) {
		if (!isset($this->articleData[$clang])) {
			$this->prewarmArticleData($clang);
		}

		if (!isset($this->articleData[$clang][$articleID])) {
			throw new Exception('Unknown article '.$articleID.'/'.$clang.' given.');
		}

		$data = $this->articleData[$clang][$articleID];

		if ($key !== null) {
			if (!array_key_exists($key, $data)) {
				throw new Exception('Invalid key "'.$key.'" given.');
			}

			return $data[$key];
		}

		return $data;
	}

	public function getArticleFragment($articleID, $clang) {
		return $this->getArticleData($articleID, $clang, 'fragment');
	}

	public function isArticleSkipped($articleID, $clang) {
		return $this->getArticleData($articleID, $clang, 'skipped');
	}

	public function addUrl($articleID, $clang, $url) {
		if (!isset($this->articleData[$clang])) {
			$this->prewarmArticleData($clang);
		}

		if (!isset($this->articleData[$clang][$articleID])) {
			throw new Exception('Unknown article '.$articleID.'/'.$clang.' given.');
		}

		$this->articleData[$clang][$articleID]['url'] = $url;
	}

	public function hasUrl($articleID, $clang) {
		if (!isset($this->articleData[$clang])) {
			$this->prewarmArticleData($clang);
		}

		return isset($this->articleData[$clang][$articleID]['url']);
	}

	public function getUrl($articleID, $clang) {
		return $this->getArticleData($articleID, $clang, 'url');
	}

	public function forgetUrl($articleID, $clang) {
		if (isset($this->articleData[$clang][$articleID])) {
			$this->articleData[$clang][$articleID]['url'] = null;
		}
	}

	public function getPathInfo(array $path, $clang) {
		$result = array();

		foreach ($path as $articleID) {
			$result[$articleID] = $this->getArticleData($articleID, $clang);
		}

		return $result;
	}
}
